@@ -2,16 +2,45 @@
 This file is modified from methylpy https://github.com/yupenghe/methylpy
 
 Author: Yupeng He
+
+Notes added on 07/03/2022 - Difference between bismark and hisat-3n
+Read mpileup doc first: http://www.htslib.org/doc/samtools-mpileup.html
+
+For Bismark SE mapping:
+Bismark converted the reads orientation based on their conversion type.
+C to T conversion reads are always map to the forward strand, regardless of the R1 R2
+G to A conversion reads are always map to the reverse strand, regardless of the R1 R2
+Therefore, in the original bam-to-allc function, we simply consider the strandness in
+the pileup format to distinguish which C needs to be counted.
+There are two situations:
+1. If the ref base is C, the read need to map to forward strand in order to be counted
+[.ATCGN] corresponding to the forward strand
+2. If the ref base is G, the read need to map to reverse strand in order to be counted
+[,atcgn] corresponding to the reverse strand
+
+For Hisat-3n PE or Biskarp PE mapping:
+R1 R2 are mapped as their original orientation, therefore,
+both the C to T and G to A conversion reads can have forward and reverse strand alignment.
+We can not distinguish the conversion type by strand in input bam.
+Here I add a check using the YZ tag of hisat-3n BAM file or XG tag of bismark BAM file.
+If the YZ tag is "+" or XG tag is "CT", the read is C to T conversion, I change the flag to forward mapping
+no matter R1 or R2 by read.is_forward = True
+If the YZ tag is "-" or XG tag is "GA", the read is G to A conversion, I change the flag to reverse mapping
+no matter R1 or R2 by read.is_forward = False
+In this case, the read orientation is the same as bismark bam file, and the following base
+count code no need to change.
+
 """
 
+import collections
 import logging
 import pathlib
 import shlex
 import subprocess
 from concurrent.futures import ProcessPoolExecutor, as_completed
 
-import collections
 import pandas as pd
+import pysam
 
 from ._doc import *
 from ._open import open_allc, open_bam
@@ -22,6 +51,40 @@ log = logging.getLogger(__name__)
 log.addHandler(logging.NullHandler())
 
 
+def _is_read_ct_conversion_hisat3n(read):
+    return read.get_tag("YZ") == "+"
+
+
+def _is_read_ct_conversion_bismark(read):
+    return read.get_tag('XG') == 'CT'
+
+
+def _convert_bam_strandness(in_bam_path, out_bam_path):
+    with pysam.AlignmentFile(in_bam_path) as in_bam, \
+            pysam.AlignmentFile(out_bam_path, header=in_bam.header, mode='wb') as out_bam:
+        is_ct_func = None
+        for read in in_bam:
+            if is_ct_func is None:
+                if read.has_tag('YZ'):
+                    is_ct_func = _is_read_ct_conversion_hisat3n
+                elif read.has_tag('XG'):
+                    is_ct_func = _is_read_ct_conversion_bismark
+                else:
+                    raise ValueError("The bam file reads has no conversion type tag "
+                                     "(XG by bismark or YZ by hisat-3n). Please note that this function can "
+                                     "only process bam files generated by bismark or hisat-3n.")
+            if is_ct_func(read):
+                read.is_forward = True
+                if read.is_paired:
+                    read.mate_is_forward = True
+            else:
+                read.is_forward = False
+                if read.is_paired:
+                    read.mate_is_forward = False
+            out_bam.write(read)
+    return
+
+
 def _read_faidx(faidx_path):
     """
     Read fadix of reference fasta file
@@ -66,19 +129,19 @@ def _get_bam_chrom_index(bam_path):
 
 
 def _bam_to_allc_worker(
-    bam_path,
-    reference_fasta,
-    fai_df,
-    output_path,
-    region=None,
-    num_upstr_bases=0,
-    num_downstr_bases=2,
-    buffer_line_number=100000,
-    min_mapq=0,
-    min_base_quality=1,
-    compress_level=5,
-    tabix=True,
-    save_count_df=False,
+        bam_path,
+        reference_fasta,
+        fai_df,
+        output_path,
+        region=None,
+        num_upstr_bases=0,
+        num_downstr_bases=2,
+        buffer_line_number=100000,
+        min_mapq=0,
+        min_base_quality=1,
+        compress_level=5,
+        tabix=True,
+        save_count_df=False,
 ):
     """
     None parallel bam_to_allc worker function, call by bam_to_allc
@@ -148,7 +211,7 @@ def _bam_to_allc_worker(
         if fields[2] not in mc_sites:
             continue
 
-        # indels
+        # deal with indels
         read_bases = fields[4]
         incons_basecalls = read_bases.count("+") + read_bases.count("-")
         if incons_basecalls > 0:
@@ -184,9 +247,10 @@ def _bam_to_allc_worker(
 
         # count converted and unconverted bases
         if fields[2] == "C":
+            # mpileup pos is 1-based, turn into 0 based
             pos = int(fields[1]) - 1
             try:
-                context = seq[(pos - num_upstr_bases) : (pos + num_downstr_bases + 1)]
+                context = seq[(pos - num_upstr_bases): (pos + num_downstr_bases + 1)]
             except:  # complete context is not available, skip
                 continue
             unconverted_c = fields[4].count(".")
@@ -195,18 +259,18 @@ def _bam_to_allc_worker(
             if cov > 0 and len(context) == context_len:
                 line_counts += 1
                 data = (
-                    "\t".join(
-                        [
-                            cur_chrom,
-                            str(pos + 1),
-                            "+",
-                            context,
-                            str(unconverted_c),
-                            str(cov),
-                            "1",
-                        ]
-                    )
-                    + "\n"
+                        "\t".join(
+                            [
+                                cur_chrom,
+                                str(pos + 1),
+                                "+",
+                                context,
+                                str(unconverted_c),
+                                str(cov),
+                                "1",
+                            ]
+                        )
+                        + "\n"
                 )
                 cov_dict[context] += cov
                 mc_dict[context] += unconverted_c
@@ -220,8 +284,8 @@ def _bam_to_allc_worker(
                     [
                         complement[base]
                         for base in reversed(
-                            seq[(pos - num_downstr_bases) : (pos + num_upstr_bases + 1)]
-                        )
+                        seq[(pos - num_downstr_bases): (pos + num_upstr_bases + 1)]
+                    )
                     ]
                 )
             except:  # complete context is not available, skip
@@ -232,18 +296,18 @@ def _bam_to_allc_worker(
             if cov > 0 and len(context) == context_len:
                 line_counts += 1
                 data = (
-                    "\t".join(
-                        [
-                            cur_chrom,
-                            str(pos + 1),
-                            "-",
-                            context,
-                            str(unconverted_c),
-                            str(cov),
-                            "1",
-                        ]
-                    )
-                    + "\n"
+                        "\t".join(
+                            [
+                                cur_chrom,
+                                str(pos + 1),
+                                "-",
+                                context,
+                                str(unconverted_c),
+                                str(cov),
+                                "1",
+                            ]
+                        )
+                        + "\n"
                 )
                 cov_dict[context] += cov
                 mc_dict[context] += unconverted_c
@@ -289,18 +353,20 @@ def _aggregate_count_df(count_dfs):
     compress_level_doc=compress_level_doc,
     cpu_basic_doc=cpu_basic_doc,
     reference_fasta_doc=reference_fasta_doc,
+    convert_bam_strandness_doc=convert_bam_strandness_doc,
 )
 def bam_to_allc(
-    bam_path,
-    reference_fasta,
-    output_path=None,
-    cpu=1,
-    num_upstr_bases=0,
-    num_downstr_bases=2,
-    min_mapq=10,
-    min_base_quality=20,
-    compress_level=5,
-    save_count_df=False,
+        bam_path,
+        reference_fasta,
+        output_path=None,
+        cpu=1,
+        num_upstr_bases=0,
+        num_downstr_bases=2,
+        min_mapq=10,
+        min_base_quality=20,
+        compress_level=5,
+        save_count_df=False,
+        convert_bam_strandness=False,
 ):
     """\
     Generate 1 ALLC file from 1 position sorted BAM file via samtools mpileup.
@@ -331,6 +397,8 @@ def bam_to_allc(
         {compress_level_doc}
     save_count_df
         If true, save an ALLC context count table next to ALLC file.
+    convert_bam_strandness
+        {convert_bam_strandness_doc}
 
     Returns
     -------
@@ -341,12 +409,21 @@ def bam_to_allc(
     tabix = True
 
     # Check fasta index
+    if not pathlib.Path(reference_fasta).exists():
+        raise FileNotFoundError(
+            f"Reference fasta not found at {reference_fasta}."
+        )
     if not pathlib.Path(reference_fasta + ".fai").exists():
         raise FileNotFoundError(
             "Reference fasta not indexed. Use samtools faidx to index it and run again."
         )
     fai_df = _read_faidx(pathlib.Path(reference_fasta + ".fai"))
 
+    if convert_bam_strandness:
+        temp_bam_path = f'{output_path}.temp.bam'
+        _convert_bam_strandness(bam_path, temp_bam_path)
+        bam_path = temp_bam_path
+
     if not pathlib.Path(bam_path + ".bai").exists():
         subprocess.check_call(shlex.split("samtools index " + bam_path))
 
@@ -389,7 +466,7 @@ def bam_to_allc(
         with ProcessPoolExecutor(max_workers=cpu) as executor:
             future_dict = {}
             for batch_id, (region, out_temp_path) in enumerate(
-                zip(regions, temp_out_paths)
+                    zip(regions, temp_out_paths)
             ):
                 _kwargs = dict(
                     bam_path=bam_path,
@@ -418,11 +495,11 @@ def bam_to_allc(
 
             # aggregate ALLC
             with open_allc(
-                output_path,
-                mode="w",
-                compresslevel=compress_level,
-                threads=1,
-                region=None,
+                    output_path,
+                    mode="w",
+                    compresslevel=compress_level,
+                    threads=1,
+                    region=None,
             ) as out_f:
                 # TODO: Parallel ALLC is overlapped,
                 #  the split by region strategy only split reads, but reads can overlap
@@ -443,9 +520,15 @@ def bam_to_allc(
             count_df = _aggregate_count_df(count_dfs)
             if save_count_df:
                 count_df.to_csv(output_path + ".count.csv")
+
+            # clean up temp bam
+            if convert_bam_strandness:
+                # this bam path is the temp file path
+                subprocess.check_call(["rm", "-f", bam_path])
+                subprocess.check_call(["rm", "-f", f'{bam_path}.bai'])
             return count_df
     else:
-        return _bam_to_allc_worker(
+        result = _bam_to_allc_worker(
             bam_path,
             reference_fasta,
             fai_df,
@@ -460,3 +543,11 @@ def bam_to_allc(
             tabix=tabix,
             save_count_df=save_count_df,
         )
+
+        # clean up temp bam
+        if convert_bam_strandness:
+            # this bam path is the temp file path
+            subprocess.check_call(["rm", "-f", bam_path])
+            subprocess.check_call(["rm", "-f", f'{bam_path}.bai'])
+
+        return result
