@@ -448,7 +448,7 @@ class ConsensusClustering:
             min_samples_split=2,
             min_samples_leaf=2,
             min_weight_fraction_leaf=0.0,
-            max_features="auto",
+            max_features="sqrt",
             max_leaf_nodes=None,
             min_impurity_decrease=0.0,
             bootstrap=True,
